/*
 * AU Factory Air Quality Sensor Firmware
 * Author: Taylor Sharpe, SweetSense Inc.
 * 
 * Microcontroller: Adafruit ItsyBitsy 32u4 3.3V
 *
 * HPMA library author Felix Galindo, @license MIT [Library altered by Taylor Sharpe]
 * Rainbow lamp function and wheel color lookup from Adafruit example code
 */

//Required HPMA Libraries
#include "Arduino.h"
#include <hpma115S0.h>

//Required NeoPixel Libraries
#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

//Global Constants
#define numLEDs 24
#define neoPin 5
#define boostPin A2
#define boardPin 13
#define modePin 7

//Use this to force a fake PM2.5 reading.
//Set to 0 for active use.
unsigned int dummyPM = 0;

//Global Variables
//Please note: This is bad programming! 
//These values should be passed through to functions instead, as
//members of a struct.
unsigned int pm2_5, pm10;
unsigned int pmMax = 1000;
unsigned int readInterval = 500;  ///Time between readings (ms)

//By default, start in PM2.5 mode
//Mode 1: PM2.5 sensor with color output; breathing animation shows a new reading
//Mode 2: Rainbow Lamp, grabbed from Adafruit source code
//Modes 3-5: Different brightness levels of white light
volatile unsigned int lampMode = 1;
volatile unsigned int intFlag = 0;

//Create a neoPixel instance
Adafruit_NeoPixel ring = Adafruit_NeoPixel(numLEDs, neoPin, NEO_GRB + NEO_KHZ800);

//Sets the brightness of LEDs when PM2.5 is being tested.  This number should be kept
//low (~150) in order to limit current draw between the lights and the sensor's fan.
int neoBrightness = 200;
int neoLowBrightness = 25;

//Arrays defining corresponding values and colors for LEDs. LEDs will change
//from blue to the specified color if they are above the corresponding neoValue.
//These values are specific to long-term risk of Accute Lower Respiratory Infection.
int neoValues[] = {2,6,20,25,35,40,50,60,70,75,85,95,110,120,130,145,160,175,195,215,240,275,310,375,500};
int neoDanger[] = {1,1, 1,  1,1,  1,1, 1, 1, 1, 1, 1,  1,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2,  2};
int neoStartup[] = {0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1};
byte neoColors[] = {145,10,255}; //Corresponds to blue, orange, red

/////LUNG CANCER ALTERNATIVE OUTPUT/////
//If the user wishes to change the LEDs to reflect risk of lung cancer instead of ALRI, this line
//can be uncommented to overwrite the ALRI values.  NOTE: The concentrations on the 
//air quality lamp cap will no longer match the readings of the LEDs!
//int neoValues[] = {2,15,30,45,60,80,100,115,140,165,190,210,240,265,295,320,350,385,415,445,480,520,555,590,630};

//Since product is not lined up with proper LED start (or direction), 
//a lookup conversion for each LED index call is required
int LEDshift[] = {5,4,3,2,1,0,23,22,21,20,19,18,17,16,15,14,13,12,11,10,9,8,7,6};

////////MODE CHANGE///////////
void modeChange(){
    if(intFlag==0){lampMode++; intFlag=1;}
    if(lampMode>=6){lampMode=1;}

    //Flicker to show button press; might also do some weird things to colors
    int currentBright = ring.getBrightness();
      for(int i = 0; i<5; i++){
      ring.setBrightness(5); ring.show(); delay(25);
      ring.setBrightness(currentBright); ring.show(); delay(25);}
}


////////////SETUP////////////////
void setup() {
  //Initialize the serial monitor
  Serial.begin(9600);  Serial.println("Starting...");

  //Initialize 5V boost pin
  pinMode(boostPin, OUTPUT);
  digitalWrite(boostPin, LOW);

  //Initialize the digital pins as outputs
  for(int pin=9; pin<=12; pin++){
    pinMode(pin, OUTPUT); digitalWrite(pin, LOW);
  }

  //Initialize the analog pins as outputs
  pinMode(A1, OUTPUT); pinMode(A2, OUTPUT);
  pinMode(A3, OUTPUT); pinMode(A4, OUTPUT);

  //Initialize the mode button
  pinMode(modePin, INPUT_PULLUP);
  //digitalWrite(modePin, HIGH);
  attachInterrupt(digitalPinToInterrupt(modePin), modeChange, FALLING);

  //Initialize the neoPixel ring and apply startupColors
  ring.begin();
  startupColors();
}

////////////MAIN LOOP////////////////
void loop() {
  switch(lampMode){
    case 1:
        delay(100); takeMeasurement(6); delay(50); 
        //Make LEDs breathe to show reading has updated
        breatheOut(5); delay(50); fillColors(); breatheIn(5); 
        fillColors(); ring.show();
          delay(readInterval);  //Wait for next reading
          
    break;
        
    case 2:
        rainbow(20);
    break;
        
    case 3:
        fillWhite(255);
    break;

    case 4:
        fillWhite(int(255*0.5));
    break;

    case 5:
        fillWhite(int(255*0.25));
    break;

      
}
//Reset button interrupt
intFlag=0;
}


///////////TAKE MEASUREMENT//////////
void takeMeasurement(float sampleSeconds){
  //Turn on the 5V Boost regulator
  digitalWrite(boostPin, HIGH); delay(100);
    //Create an instance of the hpma115S0 library
    HPMA115S0 sensor(Serial1);
    Serial1.begin(9600); delay(100);
      //Initialize the sensor
      sensor.DisableAutoSend(); delay(100);
      sensor.StartParticleMeasurement();
       //Wait for sampleSeconds, then take a measurement
       delay(sampleSeconds*1000);
        //If the read is successful, store the measured values
          if (sensor.ReadParticleMeasurement(&pm2_5, &pm10)) {
            Serial.println(pm2_5); //Allows use of the Serial Plotter
          }
          //If the read is unsuccessful, flash the on-board LED and show startup colors
          else{flashLED(boardPin); startupColors();
            Serial.println("Failed");}
          
       //If the dummyPM variable is non-zero, set pm2_5 to that value
       if(dummyPM > 0){pm2_5 = dummyPM;}
    
    //Turn off the sensor fan and close the serial port
    sensor.StopParticleMeasurement();  delay(100);
    Serial1.end();
  
  //Turn off the 5V boost pin
  digitalWrite(boostPin, LOW); delay(100);
}

///////////////FLASH LED//////////////
void flashLED(int pinNumber){
  for(int i=0; i < 10; i++){
    analogWrite(pinNumber, 255); delay(100); analogWrite(pinNumber, 0); delay(100);}
}

////////FILL WHITE//////
void fillWhite(int brightness){
    for(int i=0; i<=numLEDs; i++){
      ring.setPixelColor(LEDshift[i], 0xFFFFFF);
    }
    ring.setBrightness(brightness); ring.show(); delay(1000);
}

////////////STARTUP COLORS//////////////
void startupColors(){
    //Write colors to the LEDs corresponding to the neoValues and neoDanger arrays
    for(int i=0; i<=numLEDs; i++){ring.setPixelColor(LEDshift[i], Wheel(neoColors[neoStartup[i]]));}
    ring.setBrightness(neoBrightness); ring.show();
}

//////////////BREATHE OUT////////////
void breatheOut(int breatheTime){
  int currentBright = ring.getBrightness();

    //Breathe out
    for(int i=currentBright; i>=neoLowBrightness;i--){
          ring.setBrightness(i); ring.show(); delay(breatheTime);}
}

////////////FILL COLORS//////////////
void fillColors(){
    //Write colors to the LEDs corresponding to the neoValues and neoDanger arrays
    for(int i=0; i<=numLEDs; i++){
      //If the reading is above that LED's level, write the proper color
      if(pm2_5>neoValues[i]){ring.setPixelColor(LEDshift[i], Wheel(neoColors[neoDanger[i]]));}
      //If the reading is below the LED's level, fill it with the no-danger color
      else{ring.setPixelColor(LEDshift[i], Wheel(neoColors[0]));}
    }
    //ring.setBrightness(neoBrightness); ring.show();
}

//////////////BREATHE In////////////
void breatheIn(int breatheTime){
  
    //Breathe in
    for(int i=neoLowBrightness; i<=neoBrightness; i++){
          ring.setBrightness(i); ring.show(); delay(breatheTime);
          }
}

/////////RAINBOW ANIMATION//////////
void rainbow(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) {
    for(i=0; i<ring.numPixels(); i++) {
      ring.setPixelColor(i, Wheel((i+j) & 255));
    }
    ring.show();
    delay(wait);
  }
}

//////////////WHEEL///////////////
////Color lookup based on a color wheel input value
//Converts a value from 0 to 255 into a color from a color wheel
//Adapted from Adafruit NeoPixel example code, strandtest.ino
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {return ring.Color(255 - WheelPos * 3, 0, WheelPos * 3);}
  if(WheelPos < 170) {WheelPos -= 85; return ring.Color(0, WheelPos * 3, 255 - WheelPos * 3);}
  WheelPos -= 170;  return ring.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
